@@ -1,10 +1,11 @@
 package HTML::FormHandler::Field::Select;
 # ABSTRACT: select fields
-
+$HTML::FormHandler::Field::Select::VERSION = '0.40068';
 use Moose;
 extends 'HTML::FormHandler::Field';
 use Carp;
-our $VERSION = '0.03';
+
+use HTML::Entities;
 
 
 has 'options' => (
@@ -219,7 +220,8 @@ sub _inner_validate_field {
     }
     for my $value ( ref $value eq 'ARRAY' ? @$value : ($value) ) {
         unless ( $options{$value} ) {
-            $self->add_error($self->get_message('select_invalid_value'), $value);
+            my $opt_value = encode_entities($value);
+            $self->add_error($self->get_message('select_invalid_value'), $opt_value);
             return;
         }
     }
@@ -278,22 +280,17 @@ sub _load_options {
     return unless @options;    # so if there isn't an options method and no options
                                # from a table, already set options attributes stays put
 
-    # allow returning arrayref
-    if ( ref $options[0] eq 'ARRAY' ) {
-        @options = @{ $options[0] };
-    }
-    return unless @options;
-    my $opts;
-    # if options_<field_name> is returning an already constructed array of hashrefs
-    if ( ref $options[0] eq 'HASH' ) {
-        $opts = \@options;
+    # possibilities:
+    #  @options = ( 1 => 'one', 2 => 'two' );
+    #  @options = ([ 1 => 'one', 2 => 'tw' ]);
+    #  @options = ([ { value => 1, label => 'one'}, { value => 2, label => 'two'}]);
+    #  @options = ([[ 'one', 'two' ]]);
+    my $opts = ref $options[0] eq 'ARRAY' ? $options[0] : \@options;
+    if ( ref $opts->[0] eq 'HASH' ) {
+        # do the array of hashrefs separately so we can get the defaults
         $self->default_from_options($opts);
     }
-    else {
-        croak "Options array must contain an even number of elements for field " . $self->name
-            if @options % 2;
-        push @{$opts}, { value => shift @options, label => shift @options } while @options;
-    }
+    $opts = $self->options($opts); # coerce will reformat
     if ($opts) {
         # sort options if sort method exists
         $opts = $self->sort_options($opts) if $self->has_sort_options_method;
@@ -367,7 +364,7 @@ HTML::FormHandler::Field::Select - select fields
 
 =head1 VERSION
 
-version 0.40062
+version 0.40068
 
 =head1 DESCRIPTION
 
@@ -457,6 +454,26 @@ See also L<HTML::FormHandler::Model::DBIC>, the 'lookup_options' method.
 
 =back
 
+The options field should contain one of the following data structures:
+
+=over
+
+=item ArrayRef of HashRefs
+
+Each hash reference defines an option, with the label and value
+attributes corresponding to those of the HTML field.
+
+=item ArrayRef
+
+A list of key/value pairs corresponding to HTML field values and labels.
+
+=item ArrayRef containing one ArrayRef
+
+Each item inside the inner ArrayRef defines both the label and value of
+an option.
+
+=back
+
 =head2 Customizing options
 
 Additional attributes can be added in the options array hashref, by using
@@ -520,8 +537,8 @@ option groups.
 
 =head2 options
 
-This is an array of hashes for this field.
-Each has must have a label and value keys.
+The options available for this field as defined in the L<< /DESCRIPTION
+>> above.
 
 =head2 options_method
 
@@ -648,6 +665,11 @@ containing options:
 The select rendering widgets all have a 'render_option' method, which may be useful
 for situations when you want to split up the rendering of a radio group or checkbox group.
 
+=head2 Rendering with Template Toolkit
+
+Calling 'options' from Template Toolkit can causes issues with wantarray when there
+is only a single option. As a solution you should use 'options_ref'.
+
 =head1 Database relations
 
 Also see L<HTML::FormHandler::TraitFor::Model::DBIC>.
@@ -669,7 +691,7 @@ FormHandler Contributors - see HTML::FormHandler
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Gerda Shank.
+This software is copyright (c) 2017 by Gerda Shank.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
